/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.schemes;

import jalview.api.AlignViewportI;
import jalview.datamodel.AnnotatedCollectionI;
import jalview.datamodel.SequenceI;
import jalview.util.Comparison;

import java.awt.Color;

/**
 * A base class for colour schemes which define a graduated colour range based
 * on
 * <ul>
 * <li>a minimum colour</li>
 * <li>a maximum colour</li>
 * <li>a score assigned to each residue</li>
 * </ul>
 */
public class ScoreColourScheme extends ResidueColourScheme
{
  public double min;

  public double max;

  public double[] scores;

  /**
   * Constructor
   * 
   * @param symbolIndex
   *          a lookup where the index is a character e.g. 'R' or 'r', and the
   *          value is its position in the colour table lookup
   * @param scores
   *          per residue, indices matching colors lookup
   */
  public ScoreColourScheme(int symbolIndex[], double[] scores)
  {
    super(symbolIndex);
    setMinMax(scores);
    this.scores = scores;

    // Make colours in constructor
    // Why wasn't this done earlier?
    int iSize = scores.length;
    colors = new Color[scores.length];
    for (int i = 0; i < iSize; i++)
    {
      /*
       * scale score between min and max to the range 0.0 - 1.0
       */
      float score = (float) (scores[i] - (float) min) / (float) (max - min);

      if (score > 1.0f)
      {
        score = 1.0f;
      }

      if (score < 0.0f)
      {
        score = 0.0f;
      }
      colors[i] = makeColour(score);
    }
  }

  /**
   * Inspects score values and saves the minimum and maximum
   * 
   * @param vals
   */
  void setMinMax(double[] vals)
  {
    double dMin = Double.MAX_VALUE;
    double dMax = -Double.MAX_VALUE;

    for (int i = 0; i < vals.length - 1; i++)
    {
      dMin = Math.min(dMin, vals[i]);
      dMax = Math.max(dMax, vals[i]);
    }

    this.min = vals.length == 0 ? 0d : dMin;
    this.max = vals.length == 0 ? 0d : dMax;
  }

  @Override
  public Color findColour(char c, int j, SequenceI seq)
  {
    if (Comparison.isGap(c))
    {
      return Color.white;
    }
    return super.findColour(c);
  }

  /**
   * DOCUMENT ME!
   * 
   * @param c
   *          DOCUMENT ME!
   * 
   * @return DOCUMENT ME!
   */
  public Color makeColour(float c)
  {
    return new Color(c, (float) 0.0, (float) 1.0 - c);
  }

  @Override
  public String getSchemeName()
  {
    return "Score";
  }

  /**
   * Returns a new instance of this colour scheme with which the given data may
   * be coloured
   */
  @Override
  public ColourSchemeI getInstance(AlignViewportI view,
          AnnotatedCollectionI coll)
  {
    return new ScoreColourScheme(symbolIndex, scores);
  }
}
